@@ -39,6 +39,7 @@
 #include <qplatformdefs.h>
 #endif
 
+#include "../filenamesearch/kio_filenamesearch_p.h"
 #include "kio_sftp_debug.h"
 #include "kio_sftp_trace_debug.h"
 
@@ -47,6 +48,11 @@
 #define QT_STAT_LNK 0120000
 #endif // QT_STAT_LNK
 
+// especially windows misses CLOEXEC, so pretend it doesn't do anything
+#ifndef O_CLOEXEC
+#define O_CLOEXEC 0
+#endif
+
 using namespace KIO;
 using namespace std::filesystem;
 
@@ -1907,19 +1913,20 @@ Result SFTPWorker::mkdir(const QUrl &url, int permissions)
 
     qCDebug(KIO_SFTP_LOG) << "Trying to create directory: " << path;
     SFTPAttributesPtr sb(sftp_lstat(mSftp, path_c.constData()));
+
+    // Note: Remote end, at least in case of openssh-server
+    // applies umask. Probably most implementations do.
+    const int defaultDirectoryPermissions = S_IRWXU | S_IRWXG | S_IRWXO;
+
+    if (permissions == -1) {
+        permissions = defaultDirectoryPermissions;
+    }
     if (sb == nullptr) {
-        if (sftp_mkdir(mSftp, path_c.constData(), S_IRWXU | S_IRWXG | S_IRWXO) < 0) {
+        if (sftp_mkdir(mSftp, path_c.constData(), permissions) < 0) {
             return reportError(url, sftp_get_error(mSftp));
         }
 
         qCDebug(KIO_SFTP_LOG) << "Successfully created directory: " << url;
-        if (permissions != -1) {
-            const auto result = chmod(url, permissions);
-            if (!result.success()) {
-                return result;
-            }
-        }
-
         return Result::pass();
     }
 
